package share

import (
	"context"
	"fmt"
	"strings"

	absto "github.com/ViBiOh/absto/pkg/model"
	"github.com/ViBiOh/fibr/pkg/provider"
	"github.com/ViBiOh/httputils/v4/pkg/logger"
)

// EventConsumer handle event pushed to the event bus
func (a *App) EventConsumer(ctx context.Context, e provider.Event) {
	switch e.Type {
	case provider.RenameEvent:
		if err := a.renameItem(ctx, e.Item, *e.New); err != nil {
			logger.Error("rename share: %s", err)
		}
	case provider.DeleteEvent:
		if err := a.deleteItem(ctx, e.Item); err != nil {
			logger.Error("rename share: %s", err)
		}
	}
}

func (a *App) renameItem(ctx context.Context, old, new absto.Item) error {
	_, err := a.Exclusive(ctx, a.amqpExclusiveRoutingKey, semaphoreDuration, func(_ context.Context) error {
		for id, share := range a.shares {
			if strings.HasPrefix(share.Path, old.Pathname) {
				share.Path = strings.Replace(share.Path, old.Pathname, new.Pathname, 1)
				a.shares[id] = share

				if a.amqpClient != nil {
					if err := a.amqpClient.PublishJSON(share, a.amqpExchange, a.amqpRoutingKey); err != nil {
						return fmt.Errorf("publish share rename: %w", err)
					}
				}
			}
		}

		return provider.SaveJSON(ctx, a.storageApp, shareFilename, a.shares)
	})

	return err
}

func (a *App) deleteItem(ctx context.Context, item absto.Item) error {
	_, err := a.Exclusive(ctx, a.amqpExclusiveRoutingKey, semaphoreDuration, func(_ context.Context) error {
		for id, share := range a.shares {
			if strings.HasPrefix(share.Path, item.Pathname) {
				if err := a.delete(ctx, id); err != nil {
					return fmt.Errorf("delete share `%s`: %w", id, err)
				}
			}
		}

		return nil
	})

	return err
}
